/**HEADER********************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: msd_commands.c$
* $Version : 3.4.21.0$
* $Date    : Sep-15-2009$
*
* Comments:
*
*   This file contains device driver for mass storage class. This code tests
*   the UFI set of commands.
*
*END************************************************************************/

/**************************************************************************
Include the OS and BSP dependent files that define IO functions and
basic types. You may like to change these files for your board and RTOS 
**************************************************************************/
/**************************************************************************
Include the USB stack header files.
**************************************************************************/
#include "types.h"
#include "hostapi.h"
#include "derivative.h" /* include peripheral declarations */
#include "host_common.h"
#include "usb.h"
#include "usb_bsp.h"
#include "khci.h"
#include "usbevent.h"
#include "msd_commands.h"
#include "sci.h"
#include "rtc.h"
#include "poll.h"
#include "hidef.h"
#include "usb_host_msd_bo.h"
#include "usb_host_msd_ufi.h"
#include "usb_host_hub_sm.h"

#if (defined _MCF51MM256_H) || (defined _MCF51JE256_H)
#include "exceptions.h"
#endif


/**************************************************************************
   Global variables
**************************************************************************/
volatile DEVICE_STRUCT       mass_device = { 0 };   /* mass storage device struct */
volatile boolean bCallBack = FALSE;
volatile USB_STATUS bStatus       = USB_OK;
volatile uint_32  dBuffer_length = 0;
boolean  Attach_Notification = FALSE;
static uint_32 test = 0, ontest;
//_usb_host_handle        host_handle;         /* global handle for calling host   */


/* the following is the mass storage class driver object structure. This is
used to send commands down to  the class driver. See the Class API document
for details */

COMMAND_OBJECT_STRUCT pCmd;

/*some handles for communicating with USB stack */
uchar buff_in[BUFF_IN_SIZE], buff_out[0x0F];

void Main_Task(void);

const USB_HOST_DRIVER_INFO DriverInfoTable[] = {
   /* Floppy drive */
   {
      {0x00,0x00},                  /* Vendor ID per USB-IF             */
      {0x00,0x00},                  /* Product ID per manufacturer      */
      USB_CLASS_MASS_STORAGE,       /* Class code                       */
      USB_SUBCLASS_MASS_UFI,        /* Sub-Class code                   */
      USB_PROTOCOL_MASS_BULK,       /* Protocol                         */
      0,                            /* Reserved                         */
      usb_host_mass_device_event    /* Application call back function   */
   },

   /* USB 2.0 hard drive */
   {

      {0x49,0x0D},                  /* Vendor ID per USB-IF             */
      {0x00,0x30},                  /* Product ID per manufacturer      */
      USB_CLASS_MASS_STORAGE,       /* Class code                       */
      USB_SUBCLASS_MASS_SCSI,       /* Sub-Class code                   */
      USB_PROTOCOL_MASS_BULK,       /* Protocol                         */
      0,                            /* Reserved                         */
      usb_host_mass_device_event    /* Application call back function   */
   },

   /* USB 1.1 hub */
   {

      {0x00,0x00},                  /* Vendor ID per USB-IF             */
      {0x00,0x00},                  /* Product ID per manufacturer      */
      USB_CLASS_HUB,                /* Class code                       */
      USB_SUBCLASS_HUB_NONE,        /* Sub-Class code                   */
      USB_PROTOCOL_HUB_LS,          /* Protocol                         */
      0,                            /* Reserved                         */
      usb_host_hub_device_event     /* Application call back function   */
   },

   {
      {0x00,0x00},                  /* All-zero entry terminates        */
      {0x00,0x00},                  /*    driver info list.             */
      0,
      0,
      0,
      0,
      NULL
   }

};

#ifdef _APP_FOR_BOOT_
 typedef void (* pFun)(void);
 
void Watchdog_Reset(void)
{
  SRS = 0x55;
  SRS = 0xAA;
}

#define REDIRECT_VECTORS        0x11000
#define USER_ENTRY_ADDRESS      0x111C0
extern unsigned long far _SP_INIT[];
extern asm void _startup(void);

const byte _UserEntry[] @ USER_ENTRY_ADDRESS = {
  0x4E,
  0x71,
  0x4E,
  0xF9           //asm NOP(0x4E71), asm JMP(0x4EF9)           
};

void  (* const _UserEntry2[])()@(USER_ENTRY_ADDRESS+4)=
{
  _startup,
};

__interrupt void dummy_ISR(void) 
{
  
  
}

__interrupt void  Timer_ISR(void);
__interrupt void  USB_ISR_HOST(void);

void  (* const RAM_vector[])()@REDIRECT_VECTORS= {
    
    (pFun)&_SP_INIT,              // vector_0  INITSP
    (pFun)&_startup,             // vector_1  INITPC
    (pFun)&dummy_ISR,             // vector_2  Vaccerr
    (pFun)&dummy_ISR,             // vector_3  Vadderr
    (pFun)&dummy_ISR,             // vector_4  Viinstr
    (pFun)&dummy_ISR,             // vector_5  VReserved5
    (pFun)&dummy_ISR,             // vector_6  VReserved6
    (pFun)&dummy_ISR,             // vector_7  VReserved7
    (pFun)&dummy_ISR,             // vector_8  Vprviol
    (pFun)&dummy_ISR,             // vector_9  Vtrace
    (pFun)&dummy_ISR,             // vector_10 Vunilaop
    (pFun)&dummy_ISR,             // vector_11 Vunilfop
    (pFun)&dummy_ISR,             // vector_12 Vdbgi
    (pFun)&dummy_ISR,             // vector_13 VReserved13
    (pFun)&dummy_ISR,             // vector_14 Vferror
    (pFun)&dummy_ISR,             // vector_15 VReserved15
    (pFun)&dummy_ISR,             // vector_16 VReserved16
    (pFun)&dummy_ISR,             // vector_17 VReserved17
    (pFun)&dummy_ISR,             // vector_18 VReserved18
    (pFun)&dummy_ISR,             // vector_19 VReserved19
    (pFun)&dummy_ISR,             // vector_20 VReserved20
    (pFun)&dummy_ISR,             // vector_21 VReserved21
    (pFun)&dummy_ISR,             // vector_22 VReserved22
    (pFun)&dummy_ISR,             // vector_23 VReserved23
    (pFun)&dummy_ISR,             // vector_24 Vspuri
    (pFun)&dummy_ISR,             // vector_25 VReserved25
    (pFun)&dummy_ISR,             // vector_26 VReserved26
    (pFun)&dummy_ISR,             // vector_27 VReserved27 
    (pFun)&dummy_ISR,             // vector_28 VReserved28
    (pFun)&dummy_ISR,             // vector_29 VReserved29
    (pFun)&dummy_ISR,             // vector_30 VReserved30
    (pFun)&dummy_ISR,             // vector_31 VReserved31
    (pFun)&dummy_ISR,             // vector_32 Vtrap0
    (pFun)&dummy_ISR,             // vector_33 Vtrap1
    (pFun)&dummy_ISR,             // vector_34 Vtrap2 
    (pFun)&dummy_ISR,             // vector_35 Vtrap3
    (pFun)&dummy_ISR,             // vector_36 Vtrap4
    (pFun)&dummy_ISR,             // vector_37 Vtrap5
    (pFun)&dummy_ISR,             // vector_38 Vtrap6
    (pFun)&dummy_ISR,             // vector_39 Vtrap7
    (pFun)&dummy_ISR,             // vector_40 Vtrap8
    (pFun)&dummy_ISR,             // vector_41 Vtrap9 
    (pFun)&dummy_ISR,             // vector_42 Vtrap10
    (pFun)&dummy_ISR,             // vector_43 Vtrap11
    (pFun)&dummy_ISR,             // vector_44 Vtrap12
    (pFun)&dummy_ISR,             // vector_45 Vtrap13 
    (pFun)&dummy_ISR,             // vector_46 Vtrap14
    (pFun)&dummy_ISR,             // vector_47 Vtrap15
    (pFun)&dummy_ISR,             // vector_48 VReserved48
    (pFun)&dummy_ISR,             // vector_49 VReserved49
    (pFun)&dummy_ISR,             // vector_50 VReserved50
    (pFun)&dummy_ISR,             // vector_51 VReserved51
    (pFun)&dummy_ISR,             // vector_52 VReserved52
    (pFun)&dummy_ISR,             // vector_53 VReserved53
    (pFun)&dummy_ISR,             // vector_54 VReserved54
    (pFun)&dummy_ISR,             // vector_55 VReserved55
    (pFun)&dummy_ISR,             // vector_56 VReserved56
    (pFun)&dummy_ISR,             // vector_57 VReserved57
    (pFun)&dummy_ISR,             // vector_58 VReserved58
    (pFun)&dummy_ISR,             // vector_59 VReserved59
    (pFun)&dummy_ISR,             // vector_60 VReserved60
    (pFun)&dummy_ISR,             // vector_61 Vunsinstr
    (pFun)&dummy_ISR,             // vector_62 VReserved62
    (pFun)&dummy_ISR,             // vector_63 VReserved63
    (pFun)&dummy_ISR,             // vector_64 Virq
    (pFun)&dummy_ISR,             // vector_65 Vlvd
    (pFun)&dummy_ISR,             // vector_66 Vlol
    (pFun)&dummy_ISR,             // vector_67 Vspi1
    (pFun)&dummy_ISR,             // vector_68 Vspi2 
    (pFun)&USB_ISR_HOST,               // vector_69 Vusb
    (pFun)&dummy_ISR,             // vector_70 VReserved70
    (pFun)&dummy_ISR,             // vector_71 Vtpm1ch0
    (pFun)&dummy_ISR,             // vector_72 Vtpm1ch1
    (pFun)&dummy_ISR,             // vector_73 Vtpm1ch2
    (pFun)&dummy_ISR,             // vector_74 Vtpm1ch3 
    (pFun)&dummy_ISR,             // vector_75 Vtpm1ch4
    (pFun)&dummy_ISR,             // vector_76 Vtpm1ch5
    (pFun)&dummy_ISR,             // vector_77 Vtpm1ovf
    (pFun)&dummy_ISR,             // vector_78 Vtpm2ch0
    (pFun)&dummy_ISR,             // vector_79 Vtpm2ch1
    (pFun)&dummy_ISR,             // vector_80 Vtpm2ovf
    (pFun)&dummy_ISR,             // vector_81 Vsci1err
    (pFun)&dummy_ISR,             // vector_82 Vsci1rx
    (pFun)&dummy_ISR,             // vector_83 Vsci1tx
    (pFun)&dummy_ISR,             // vector_84 Vsci2err
    (pFun)&dummy_ISR,             // vector_85 Vsci2rx
    (pFun)&dummy_ISR,             // vector_86 Vsci2tx
    (pFun)&dummy_ISR,             // vector_87 Vkeyboard
    (pFun)&dummy_ISR,             // vector_88 Vadc
    (pFun)&dummy_ISR,             // vector_89 Vacmpx
    (pFun)&dummy_ISR,             // vector_90 Viic1x
    (pFun)&Timer_ISR,             // vector_91 Vrtc
    (pFun)&dummy_ISR,             // vector_92 Viic2x
    (pFun)&dummy_ISR,             // vector_93 Vcmt
    (pFun)&dummy_ISR,             // vector_94 Vcanwu
    (pFun)&dummy_ISR,             // vector_95 Vcanerr 
    (pFun)&dummy_ISR,             // vector_96 Vcanrx
    (pFun)&dummy_ISR,             // vector_97 Vcantx
    (pFun)&dummy_ISR,             // vector_98 Vrnga
    (pFun)&dummy_ISR,             // vector_99 VReserved99
    (pFun)&dummy_ISR,             // vector_100 VReserved100
    (pFun)&dummy_ISR,             // vector_101 VReserved101 
    (pFun)&dummy_ISR,             // vector_102 VReserved102
    (pFun)&dummy_ISR,             // vector_103 VReserved103
    (pFun)&dummy_ISR,             // vector_104 VL7swi 
    (pFun)&dummy_ISR,             // vector_105 VL6swi
    (pFun)&dummy_ISR,             // vector_106 VL5swi
    (pFun)&dummy_ISR,             // vector_107 VL4swi
    (pFun)&dummy_ISR,             // vector_108 VL3swi
    (pFun)&dummy_ISR,             // vector_109 VL2swi
    (pFun)&dummy_ISR,             // vector_110 VL1swi
};
#endif

/*FUNCTION*----------------------------------------------------------------
*
* Function Name  : main
* Returned Value : none
* Comments       :
*     Execution starts here
*
*END*--------------------------------------------------------------------*/
void main(uint_32 param) 
{
  USB_STATUS           status = USB_OK;
  _usb_host_handle     host_handle;
  
  UNUSED(param)

#ifdef _APP_FOR_BOOT_  
  /* !! This section needs to be here to redirect interrupt vectors !! */
  dword *pdst,*psrc;
  byte i;
  
  asm (move.l  #0x00800000,d0);
  asm (movec   d0,vbr);
  
  pdst=(dword*)0x00800000;
  psrc=(dword*)&RAM_vector;
  
  for (i=0;i<111;i++,pdst++,psrc++)
  {
    *pdst=*psrc;
  }
#endif

  /* Initialize the current platform. Call for the _bsp_platform_init which is specific to each processor family */
  _bsp_platform_init();
  sci1_init();
  
  TimerInit();
  
  /* Init polling global variable */
  POLL_init(); 

  DisableInterrupts;
  #if (defined _MCF51MM256_H) || (defined _MCF51JE256_H)
  usb_int_dis();
  #endif  
  status = _usb_host_init
    (HOST_CONTROLLER_NUMBER,    /* Use value in header file */
     MAX_FRAME_SIZE,            /* Frame size per USB spec  */
     &host_handle);             /* Returned pointer */

  if (status != USB_OK) {
    printf("\nUSB Host Initialization failed. STATUS: %x", status);    
  } /* Endif */

   /*
   ** Since we are going to act as the host driver, register
   ** the driver information for wanted class/subclass/protocols
   */
  status = _usb_host_driver_info_register(host_handle, (void *)DriverInfoTable);
  if (status != USB_OK) {
     printf("\nDriver Registration failed. STATUS: %x", status);      
  }
   
  EnableInterrupts;
  #if (defined _MCF51MM256_H) || (defined _MCF51JE256_H)
  usb_int_en();
  #endif
   
  pCmd.CBW_PTR = (CBW_STRUCT_PTR) malloc(sizeof(CBW_STRUCT));
  
  if (pCmd.CBW_PTR == NULL)
  {
     printf ("\nUnable to allocate Command Block Wrapper");
     
  }
  memset(pCmd.CBW_PTR, 0, sizeof(CBW_STRUCT));
   
  pCmd.CSW_PTR = (CSW_STRUCT_PTR) malloc(sizeof(CSW_STRUCT));
   
  if (pCmd.CSW_PTR == NULL)
  {
     printf ("\nUnable to allocate Command Status Wrapper");
     USB_mem_free(pCmd.CBW_PTR);
  }
    
  memset(pCmd.CSW_PTR, 0, sizeof(CSW_STRUCT));
  
  printf("\nUSB MSD Command test\nWaiting for USB mass storage to be attached...\n");

  for(;;) {
    Main_Task();
    Poll();
    __RESET_WATCHDOG(); /* feeds the dog */
  } /* loop forever */
  /* please make sure that you never leave main */
 
}
/*FUNCTION*----------------------------------------------------------------
*
* Function Name  : main
* Returned Value : none
* Comments       :
*     Execution starts here
*
*END*--------------------------------------------------------------------*/

void Main_Task ( void )
{ /* Body */
   USB_STATUS           status = USB_OK;



   /*----------------------------------------------------**
   ** Infinite loop, waiting for events requiring action **
   **----------------------------------------------------*/
      switch ( mass_device.dev_state ) {
         case USB_DEVICE_IDLE:
            break ;
         case USB_DEVICE_ATTACHED:
            printf( "Mass Storage Device Attached\n" );
            mass_device.dev_state = USB_DEVICE_SET_INTERFACE_STARTED;
            status = _usb_hostdev_select_interface(mass_device.dev_handle,
                 mass_device.intf_handle, (pointer)&mass_device.class_intf);
            break ;
         case USB_DEVICE_SET_INTERFACE_STARTED:
            break;
         case USB_DEVICE_INTERFACED:
            usb_host_mass_test_storage();
            //mass_device.dev_state = USB_DEVICE_OTHER;
            break ;
         case USB_DEVICE_DETACHED:
            printf ( "\nMass Storage Device Detached\n" );
            mass_device.dev_state = USB_DEVICE_IDLE;
            break;
         case USB_DEVICE_OTHER:
            break ;
         default:
            printf ( "Unknown Mass Storage Device State = %d\n",\
               mass_device.dev_state );
            break ;
      } /* Endswitch */
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
*
* Function Name  : usb_host_mass_device_event
* Returned Value : None
* Comments       :
*     called when mass storage device has been attached, detached, etc.
*END*--------------------------------------------------------------------*/
void usb_host_mass_device_event
   (
      /* [IN] pointer to device instance */
      _usb_device_instance_handle      dev_handle,

      /* [IN] pointer to interface descriptor */
      _usb_interface_descriptor_handle intf_handle,

      /* [IN] code number for event causing callback */
      uint_32           event_code
   )
{ /* Body */
   INTERFACE_DESCRIPTOR_PTR   intf_ptr =
      (INTERFACE_DESCRIPTOR_PTR)intf_handle;

   switch (event_code) {
      case USB_CONFIG_EVENT:
         /* Drop through into attach, same processing */
      case USB_ATTACH_EVENT:
         if (mass_device.dev_state == USB_DEVICE_IDLE) {
            mass_device.dev_handle = dev_handle;
            mass_device.intf_handle = intf_handle;
            mass_device.dev_state = USB_DEVICE_ATTACHED;
         } else {
            printf("Mass Storage Device Is Already Attached\n");
         } /* EndIf */
         break;
      case USB_INTF_EVENT:
         mass_device.dev_state = USB_DEVICE_INTERFACED;
         break;
      case USB_DETACH_EVENT:
    	  test = 0;
    	  if (mass_device.dev_state != USB_DEVICE_IDLE) 
         {
            mass_device.dev_handle = NULL;
            mass_device.intf_handle = NULL;
            mass_device.dev_state = USB_DEVICE_DETACHED;
         } 
         else 
         {
            printf("Mass Storage Device Is Not Attached\n");
         } /* EndIf */
         break;
      default:
         mass_device.dev_state = USB_DEVICE_IDLE;
         break;
   } /* EndSwitch */
} /* Endbody */


/*FUNCTION*----------------------------------------------------------------
*
* Function Name  : usb_host_mass_ctrl_calback
* Returned Value : None
* Comments       :
*     called on completion of a control-pipe transaction.
*
*END*--------------------------------------------------------------------*/

static void usb_host_mass_ctrl_callback
   (
      /* [IN] pointer to pipe */
      _usb_pipe_handle  pipe_handle,

      /* [IN] user-defined parameter */
      pointer           user_parm,

      /* [IN] buffer address */
      uchar_ptr         buffer,

      /* [IN] length of data transferred */
      uint_32           buflen,

      /* [IN] status, hopefully USB_OK or USB_DONE */
      uint_32           status
   )
{ /* Body */

    UNUSED(pipe_handle)
    UNUSED(user_parm)
    UNUSED(buffer)
    UNUSED(buflen)
    
   bCallBack = TRUE;
   bStatus = status;

} /* Endbody */


void callback_bulk_pipe
   (
      /* [IN] Status of this command */
      USB_STATUS status,

      /* [IN] pointer to USB_MASS_BULK_ONLY_REQUEST_STRUCT*/
      pointer p1,

      /* [IN] pointer to the command object*/
      pointer  p2,

      /* [IN] Length of data transmitted */
      uint_32 buffer_length
   )
{ /* Body */

    UNUSED(p1)
    UNUSED(p2)
    
   dBuffer_length = buffer_length;
   bCallBack = TRUE;
   bStatus = status;

} /* Endbody */



/*FUNCTION*----------------------------------------------------------------
*
* Function Name  : usb_host_mass_test_storage
* Returned Value : None
* Comments       :
*     Calls the UFI command set for testing
*END*--------------------------------------------------------------------*/
static void usb_host_mass_test_storage
   (
      void
   )
{ /* Body */
   static USB_STATUS                                 status = USB_OK;
   static uint_8                                     bLun = 0;
   static INQUIRY_DATA_FORMAT                        inquiry;
   static CAPACITY_LIST                              capacity_list;
   static MASS_STORAGE_READ_CAPACITY_CMD_STRUCT_INFO read_capacity;
   static REQ_SENSE_DATA_FORMAT                      req_sense;
   static FORMAT_UNIT_PARAMETER_BLOCK                formatunit = { 0};
   static CBW_STRUCT_PTR cbw_ptr; 
   static CSW_STRUCT_PTR csw_ptr;
   switch(test){
    case 0:
      cbw_ptr = pCmd.CBW_PTR;
      csw_ptr = pCmd.CSW_PTR;

      memset(buff_in, 0, BUFF_IN_SIZE);
      memset(buff_out, 0, 0x0F);
      memset(pCmd.CSW_PTR, 0, sizeof(CSW_STRUCT));
      memset(pCmd.CBW_PTR, 0, sizeof(CBW_STRUCT));
      memset(&pCmd, 0, sizeof(COMMAND_OBJECT_STRUCT));

      pCmd.CBW_PTR  = cbw_ptr;
      pCmd.CSW_PTR  = csw_ptr;
      pCmd.LUN      = bLun;
      pCmd.CALL_PTR = (pointer)&mass_device.class_intf;
      pCmd.CALLBACK = callback_bulk_pipe;

      printf("\n =============START OF A NEW SESSION==================");
      test++;
      ontest = 1;
      break;
    case 1:
      /* Test the GET MAX LUN command */
      if (1 == ontest){
        printf("\nTesting: GET MAX LUN Command");
        bCallBack = FALSE;

        status = usb_class_mass_getmaxlun_bulkonly(
          (pointer)&mass_device.class_intf, &bLun,
          usb_host_mass_ctrl_callback);
          ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n....................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("....................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }

        }
      }
      break;
    case 2:
      /* Test the TEST UNIT READY command */
      if (1 == ontest){
        printf("Testing: TEST UNIT READY Command");
        bCallBack = FALSE;

        status =  usb_mass_ufi_test_unit_ready(&pCmd);
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported(bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 3:
      /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: REQUEST SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_request_sense(&pCmd, &req_sense,
          sizeof(REQ_SENSE_DATA_FORMAT));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 4:
      /* Test the INQUIRY command */
      if (1 == ontest){
        printf("Testing: INQUIRY Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_inquiry(&pCmd, (uchar_ptr) &inquiry,
          sizeof(INQUIRY_DATA_FORMAT));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n........................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("........................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 5:
      /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: REQUEST SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_request_sense(&pCmd, &req_sense,
          sizeof(REQ_SENSE_DATA_FORMAT));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 6:
      /* Test the READ FORMAT CAPACITY command */
      if (1 == ontest){
        printf("Testing: READ FORMAT CAPACITIES Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_format_capacity(&pCmd, (uchar_ptr)&capacity_list,
          sizeof(CAPACITY_LIST));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n.........ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf(".........OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 7:
      /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: REQUEST SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_request_sense(&pCmd, &req_sense,
          sizeof(REQ_SENSE_DATA_FORMAT));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 8:
      /* Test the READ CAPACITY command */
      if (1 == ontest){
        printf("Testing: READ CAPACITY Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_read_capacity(&pCmd, (uchar_ptr)&read_capacity,
          sizeof(MASS_STORAGE_READ_CAPACITY_CMD_STRUCT_INFO));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 9:
      /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: REQUEST SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_request_sense(&pCmd, &req_sense,
          sizeof(REQ_SENSE_DATA_FORMAT));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 10:
      /* Test the READ(10) command */
      if (1 == ontest){
        printf("Testing: READ(10) Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_read_10(&pCmd, 0, buff_in, 512, 1);
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n.......................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf(".......................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 11:
      /* Test the MODE SENSE command */
      if (1 == ontest){
        printf("Testing: MODE SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_mode_sense(&pCmd,
          2, //PC
          0x3F, //page code
          buff_in,
          (uint_32)0x08);
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n.....................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf(".....................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 12:
      /* Test the PREVENT ALLOW command */
      if (1 == ontest){
        printf("Testing: PREVENT-ALLOW MEDIUM REMOVAL Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_prevent_allow_medium_removal(
          &pCmd,
          1 // prevent
          );
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n...ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("...OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 13:
      /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: REQUEST SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_request_sense(&pCmd, &req_sense,
          sizeof(REQ_SENSE_DATA_FORMAT));
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 14:
       /* Test the VERIFY command */
      if (1 == ontest){
        printf("Testing: VERIFY Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_verify(
          &pCmd,
          0x400, // block address
          1 //length to be verified
          );
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n.........................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf(".........................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 15:
       /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: WRITE(10) Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_write_10(&pCmd, 8, buff_out, 512, 1);
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n......................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("......................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          ;
        }
      }
      break;
    case 16:
       /* Test the REQUEST SENSE command */
      if (1 == ontest){
        printf("Testing: REQUEST SENSE Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_write_10(&pCmd, 8, buff_out, 512, 1);
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n..................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("..................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 17:
       /* Test the START-STOP UNIT command */
      if (1 == ontest){
        printf("Testing: START-STOP UNIT Command");
        bCallBack = FALSE;

        status = usb_mass_ufi_start_stop(&pCmd, 0, 1);
        ontest = 0;
      } else {
        if ((status != USB_OK) && (status != USB_STATUS_TRANSFER_QUEUED))
        {
          printf ("\n................ERROR");
          return;
        }
        else
        {
          /* Wait till command comes back */
          if (!bCallBack) {break;}
          if (!bStatus) {
             printf("................OK\n");
             test++;
             ontest = 1;
          }
          else {
             printf("...Unsupported (bStatus=0x%d)\n", bStatus);
             test++;
             ontest = 1;
          }
          
        }
      }
      break;
    case 18:
      printf("\nTest done!\n");
      test++;
      break;
    default:
      break;
   } /* End of switch */

} /* Endbody */



/* EOF */